using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Ants
{
    public class Path
    {
        public PathTarget Target
        {
            get { return fTarget; }
            set { fTarget = value; }
        }

        public double Length
        {
            get
            {
                return fLength;
            }
        }

        public PointF this[int i]
        {
            get { return fSteps[i]; }
        }

        public RectangleF Bounds
        {
            get { return fBounds; }
        }

        public enum PathTarget
        {
            Nothing = 0,
            Food = 1
        }

        PointF[] fSteps;
        int fLength;
        PathTarget fTarget = PathTarget.Nothing;
        RectangleF fBounds;

        public Path(int aMaxLength)
        {
            fSteps = new PointF[aMaxLength];
            fLength = 0;
            fBounds.X = -1;

            for (int i = 0; i < aMaxLength; ++i)
                fSteps[i] = new PointF(0, 0);
        }

        public Path Clone()
        {
            Path lOut = new Path(fSteps.Length);

            lOut.fLength = this.fLength;
            lOut.fTarget = this.fTarget;

            for (int i = 0; i < fLength; ++i)
            {
                lOut.fSteps[i] = new PointF(fSteps[i].X, fSteps[i].Y);
            }

            return lOut;
        }

        public void AddStep(PointF aStep)
        {
            fSteps[fLength] = new PointF(aStep.X, aStep.Y);

            ++fLength;

            if (fBounds.X < 1)
            {
                fBounds = new RectangleF(aStep.X, aStep.Y, 0, 0);
            }
            else
            {
                if (aStep.X < fBounds.X)
                    fBounds.X = aStep.X;

                if (aStep.Y < fBounds.Y)
                    fBounds.Y = aStep.Y;

                if (aStep.X > fBounds.Right)
                    fBounds.Width = aStep.X - fBounds.X;

                if (aStep.Y > fBounds.Bottom)
                    fBounds.Height = aStep.Y - fBounds.Y;
            }
        }

        public void Clear()
        {
            fLength = 0;
        }

    }
}
