using System;
using System.Collections.Generic;
using System.Windows.Forms;
using System.IO;
using System.Drawing;
using System.Text;

namespace Ants
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main(string[] args)
        {
            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);

            //create parser
            CMDLineParser parser = new CMDLineParser();

            //Add options for input
            CMDLineParser.NumberOption NumCol = parser.AddIntParameter("-c",
                                     "Number of colonies", true);
            CMDLineParser.NumberOption NumNest = parser.AddIntParameter("-n",
                                    "Number of nests per colony", true);
            CMDLineParser.NumberOption NumForager = parser.AddIntParameter("-f",
                                    "Number of foragers per nest", true);
            CMDLineParser.NumberOption NumTransporter = parser.AddIntParameter("-t",
                                    "Number of transporters per nest", true);
            CMDLineParser.NumberOption NumReps = parser.AddIntParameter("-r",
                                    "Number of replicates", true);
            CMDLineParser.NumberOption RecruitEmptyNests = parser.AddIntParameter("-e",
                                    "Recruitment to empty nests", true);
            CMDLineParser.NumberOption RouteFidelity = parser.AddIntParameter("-F",
                        "Ants have route fidelity", true);
            CMDLineParser.Option NestFile = parser.AddStringParameter("-I",
                                    "Input file containing nest locations", false);
            try
            {
                //parse the command line
                parser.Parse(args);
            }
            catch (CMDLineParser.CMDLineParserException ex)
            {
                //show available options
                System.Windows.Forms.MessageBox.Show("Error: " + ex.Message);
                System.Windows.Forms.MessageBox.Show(parser.HelpMessage());
                return;
            }

            //replace argument list with remaining arguments
            args = parser.RemainingArgs();

            //Get values 
            int aNests = (int)NumNest.Value;
            int aReps = (int)NumReps.Value;
            int aColonies = (int)NumCol.Value;
            int aForagers = (int)NumForager.Value;
            int aTransporters = (int)NumTransporter.Value;
            string aNestFile = (string)NestFile.Value;
            bool aEmptyRecruit;
            if ((int)RecruitEmptyNests.Value == 0)
            {
                aEmptyRecruit = false;
            }
            else
            {
               aEmptyRecruit = true;
            }
            bool aRouteFidelity;
            if ((int)RouteFidelity.Value == 0)
            {
                aRouteFidelity = false;
            }
            else
            {
                aRouteFidelity = true;
            }

            //Fixed values
            int aRunLength = 30000;
            int aTotalFood = 1000000;

            //program will loop for each food number in this list
            List<int> aFoods = new List<int>();
            aFoods.Add(3);

            foreach (int aFood in aFoods)
            {
                //Create world
                World world1 = new World(aTotalFood, aFood, aColonies, aNests, aForagers, aTransporters, aNestFile, aEmptyRecruit, aRouteFidelity);
                //Create output
                Output fOutput = new Output(world1, aFood, aNests, aForagers);
                if (aNestFile != null)
                {
                    fOutput.WriteDistances(world1, "");
                }

                for (int i = 0; i < aReps; ++i)
                {
                    RunRep(world1, aRunLength, fOutput, aNestFile, i, null);
                }
                //output at end of all reps
                fOutput.EndAll();
            }
        }

        public static void RunRep(World aWorld, int aRunLength, Output aOutput, string aNestFile, int RepNum, Graphics graphics)
        {
            for (int j = 0; j <= aRunLength; j++)
            {
                RunStep(aWorld, aOutput, j, RepNum, graphics);
            }

            //output at end of every rep
            aOutput.EndRep(aWorld, RepNum);
            string FileName = "Rep" + RepNum;
            aOutput.WriteNestVisitors(aWorld, FileName);
            if (aNestFile == null)
            {
                aOutput.WriteDistances(aWorld, FileName);
            }
            aWorld.Restart();
        }

        public static void RunStep(World aWorld, Output aOutput, int StepNum, int RepNum, Graphics graphics)
        {
            aWorld.Step(graphics);
            //periodic output 
            if (StepNum % 500 == 0 && StepNum > 0)
            {
                string File1 = "Rep_" + RepNum + "Time_" + StepNum;
                //Maps output less often
                if (StepNum % 10000 == 0)
                {
                    aOutput.DrawMap(aWorld, true, true, true, true, File1);
                }

                //INFO: Temporary food added here;
                //int NumberTempFoods = 100;
                //int EnergyPerTempFood = 5;
                //aWorld.AddTempFoods(NumberTempFoods, EnergyPerTempFood);  
                aOutput.WriteMovement(aWorld, File1);
                aWorld.ResetMovement();
            }
        }
    }
}
